@@ -17,7 +17,7 @@ from dexp.datasets.operations.deconv import dataset_deconv
 @click.option('--codec', '-z', default=_default_codec, help='compression codec: ‘zstd’, ‘blosclz’, ‘lz4’, ‘lz4hc’, ‘zlib’ or ‘snappy’ ', show_default=True)
 @click.option('--clevel', '-l', type=int, default=_default_clevel, help='Compression level', show_default=True)
 @click.option('--overwrite', '-w', is_flag=True, help='to force overwrite of target', show_default=True)
-@click.option('--chunksize', '-cs', type=int, default=512, help='Chunk size for tiled computation', show_default=True)
+@click.option('--tilesize', '-ts', type=int, default=512, help='Tile size for tiled computation', show_default=True)
 @click.option('--method', '-m', type=str, default='lr', help='Deconvolution method: for now only lr (Lucy Richardson)', show_default=True)
 @click.option('--iterations', '-i', type=int, default=None, help='Number of deconvolution iterations. More iterations takes longer, will be sharper, but might also be potentially more noisy depending on method. '
                                                                  'The default number of iterations depends on the other parameters, in particular it depends on the choice of backprojection operator. For ‘wb’ as little as 3 iterations suffice. ',
@@ -47,7 +47,7 @@ def deconv(input_paths,
            codec,
            clevel,
            overwrite,
-           chunksize,
+           tilesize,
            method,
            iterations,
            maxcorrection,
@@ -86,20 +86,20 @@ def deconv(input_paths,
                        compression=codec,
                        compression_level=clevel,
                        overwrite=overwrite,
-                       chunksize=chunksize,
+                       tilesize=tilesize,
                        method=method,
                        num_iterations=iterations,
                        max_correction=maxcorrection,
                        power=power,
                        blind_spot=blindspot,
                        back_projection=backprojection,
-                       objective=objective,
-                       numerical_aperture=numape,
-                       dxy=dxy,
-                       dz=dz,
-                       xy_size=xysize,
-                       z_size=zsize,
-                       show_psf=showpsf,
+                       psf_objective=objective,
+                       psf_na=numape,
+                       psf_dxy=dxy,
+                       psf_dz=dz,
+                       psf_xy_size=xysize,
+                       psf_z_size=zsize,
+                       psf_show=showpsf,
                        scaling=scaling,
                        workers=workers,
                        workersbackend=workersbackend,
