@@ -53,6 +53,7 @@ def cli():
 from dexp.cli.dexp_commands.info import info
 from dexp.cli.dexp_commands.check import check
 from dexp.cli.dexp_commands.copy import copy
+from dexp.cli.dexp_commands.fastcopy import fastcopy
 from dexp.cli.dexp_commands.add import add
 from dexp.cli.dexp_commands.tiff import tiff
 from dexp.cli.dexp_commands.view import view
@@ -60,6 +61,7 @@ from dexp.cli.dexp_commands.serve import serve
 
 from dexp.cli.dexp_commands.deskew import deskew
 from dexp.cli.dexp_commands.fuse import fuse
+from dexp.cli.dexp_commands.register import register
 from dexp.cli.dexp_commands.deconv import deconv
 from dexp.cli.dexp_commands.stabilize import stabilize
 from dexp.cli.dexp_commands.isonet import isonet
@@ -72,6 +74,7 @@ from dexp.cli.dexp_commands.projrender import projrender
 cli.add_command(info)
 cli.add_command(check)
 cli.add_command(copy)
+cli.add_command(fastcopy)
 cli.add_command(add)
 cli.add_command(tiff)
 cli.add_command(view)
@@ -79,6 +82,7 @@ cli.add_command(serve)
 
 cli.add_command(deskew)
 cli.add_command(fuse)
+cli.add_command(register)
 cli.add_command(stabilize)
 cli.add_command(deconv)
 cli.add_command(isonet)
