@@ -1,16 +1,17 @@
-from typing import Sequence, List
-
+import dask
+import numpy as np
 from arbol.arbol import aprint
 from arbol.arbol import asection
-from joblib import Parallel, delayed
-from zarr.errors import ContainsArrayError, ContainsGroupError
+from dask.distributed import Client
 
+from dexp.datasets.zarr_dataset import ZDataset
 from dexp.processing.backends.backend import Backend
 from dexp.processing.backends.best_backend import BestBackend
 from dexp.processing.backends.numpy_backend import NumpyBackend
 from dexp.processing.multiview_lightsheet.fusion.mvsols import msols_fuse_1C2L
-from dexp.processing.multiview_lightsheet.fusion.simview import simview_fuse_2C2L
+from dexp.processing.multiview_lightsheet.fusion.simview import SimViewFusion
 from dexp.processing.registration.model.model_io import model_list_from_file, model_list_to_file
+from dexp.utils.slicing import slice_from_shape
 
 
 def dataset_fuse(dataset,
@@ -33,10 +34,12 @@ def dataset_fuse(dataset,
                  dark_denoise_threshold,
                  z_pad_apodise,
                  loadreg,
+                 model_list_filename,
                  warpreg_num_iterations,
                  min_confidence,
                  max_change,
                  registration_edge_filter,
+                 maxproj,
                  huge_dataset,
                  workers,
                  workersbackend,
@@ -44,75 +47,64 @@ def dataset_fuse(dataset,
                  check,
                  stop_at_exception=True):
 
-    views = tuple(dataset.get_array(channel, per_z_slice=False, wrap_with_dask=True) for channel in channels)
+    views = tuple(dataset.get_array(channel, per_z_slice=False) for channel in channels)
 
-    with asection(f"views:"):
+    with asection(f"Views:"):
         for view, channel in zip(views, channels):
             aprint(f"View: {channel} of shape: {view.shape} and dtype: {view.dtype}")
 
-    if slicing is not None:
-        aprint(f"Slicing with: {slicing}")
-        views = tuple(view[slicing] for view in views)
-
-    # shape and dtype of views to fuse:
-    shape = views[0].shape
     dtype = views[0].dtype
-    nb_timepoints = shape[0]
-
-    # We allocate last minute once we know the shape...
-    from dexp.datasets.zarr_dataset import ZDataset
-    mode = 'w' + ('' if overwrite else '-')
-    dest_dataset = ZDataset(output_path, mode, store)
+    in_nb_time_points = views[0].shape[0]
+    aprint(f"Slicing with: {slicing}")
+    out_shape, volume_slicing, time_points = slice_from_shape(views[0].shape, slicing)
 
     # load registration models:
     with NumpyBackend():
-        model_list_filename = "registration_models.txt"
         if loadreg:
             aprint(f"Loading registration shifts from existing file! ({model_list_filename})")
             models = model_list_from_file(model_list_filename)
+            if len(models) == 1:
+                models = [models[0] for _ in range(in_nb_time_points)]
+            elif len(models) != in_nb_time_points:
+                raise ValueError(f'Number of registration models provided ({len(models)})'
+                                 f'differs from number of input time points ({in_nb_time_points})')
         else:
-            models = [None, ] * nb_timepoints
+            models = [None] * len(time_points)
 
-    # hold equalisation ratios:
-    equalisation_ratios_reference: List[Sequence[float]] = [[]]
-    if microscope == 'simview':
-        equalisation_ratios_reference[0] = [None, None, None]
-    elif microscope == 'mvsols':
-        equalisation_ratios_reference[0] = [None, ]
-
-    def process(tp, device, workers):
+    @dask.delayed
+    def process(i, params):
+        equalisation_ratios_reference, model, dest_dataset = params
+        tp = time_points[i]
         try:
+            with asection(f"Loading channels {channels} for time point {i}/{len(time_points)}"):
+                views_tp = tuple(np.asarray(view[tp][volume_slicing]) for view in views)
 
-            with asection(f"Loading channels {channels} for time point {tp}/{nb_timepoints}"):
-                views_tp = tuple(view[tp].compute() for view in views)
-
-            with BestBackend(device, exclusive=True, enable_unified_memory=True):
-
-                model = models[tp]
-
-                # If we don't have a model for that timepoint we load one from a previous timepoint
-                if model is None and tp >= workers:
-                    aprint(f"we don't have a registration model for timepoint {tp}/{nb_timepoints} so we load one from a previous timepoint: {tp - workers}")
-                    model = models[tp - workers]
+            with BestBackend(exclusive=True, enable_unified_memory=True):
+                if models[i] is not None:
+                    model = models[i]
+                # otherwise it could be a None model or from the first iteration if equalisation mode was 'first'
 
                 if microscope == 'simview':
-                    tp_array, model, new_equalisation_ratios = simview_fuse_2C2L(*views_tp,
-                                                                                 registration_force_model=loadreg,
-                                                                                 registration_model=model,
-                                                                                 registration_min_confidence=min_confidence,
-                                                                                 registration_max_change=max_change,
-                                                                                 registration_edge_filter=registration_edge_filter,
-                                                                                 equalise=equalise,
-                                                                                 equalisation_ratios=equalisation_ratios_reference[0],
-                                                                                 zero_level=zero_level,
-                                                                                 clip_too_high=clip_too_high,
-                                                                                 fusion=fusion,
-                                                                                 fusion_bias_exponent=2,
-                                                                                 fusion_bias_strength_i=fusion_bias_strength_i,
-                                                                                 fusion_bias_strength_d=fusion_bias_strength_d,
-                                                                                 dehaze_size=dehaze_size,
-                                                                                 dark_denoise_threshold=dark_denoise_threshold,
-                                                                                 huge_dataset_mode=huge_dataset)
+                    fuse_obj = SimViewFusion(registration_model=model,
+                                             equalise=equalise,
+                                             equalisation_ratios=equalisation_ratios_reference,
+                                             zero_level=zero_level,
+                                             clip_too_high=clip_too_high,
+                                             fusion=fusion,
+                                             fusion_bias_exponent=2,
+                                             fusion_bias_strength_i=fusion_bias_strength_i,
+                                             fusion_bias_strength_d=fusion_bias_strength_d,
+                                             dehaze_before_fusion=True,
+                                             dehaze_size=dehaze_size,
+                                             dehaze_correct_max_level=True,
+                                             dark_denoise_threshold=dark_denoise_threshold,
+                                             dark_denoise_size=9,
+                                             butterworth_filter_cutoff=1,
+                                             flip_camera1=True)
+
+                    tp_array = fuse_obj(*views_tp)
+                    new_equalisation_ratios = fuse_obj._equalisation_ratios
+
                 elif microscope == 'mvsols':
                     metadata = dataset.get_metadata()
                     angle = metadata['angle']
@@ -131,7 +123,7 @@ def dataset_fuse(dataset,
                                                                                registration_max_change=max_change,
                                                                                registration_edge_filter=registration_edge_filter,
                                                                                equalise=equalise,
-                                                                               equalisation_ratios=equalisation_ratios_reference[0],
+                                                                               equalisation_ratios=equalisation_ratios_reference,
                                                                                zero_level=zero_level,
                                                                                clip_too_high=clip_too_high,
                                                                                fusion=fusion,
@@ -143,65 +135,80 @@ def dataset_fuse(dataset,
                                                                                dx=res,
                                                                                dz=dz,
                                                                                illumination_correction_sigma=illumination_correction_sigma,
+                                                                               registration_mode='projection' if maxproj else 'full',
                                                                                huge_dataset_mode=huge_dataset)
+                else:
+                    raise NotImplementedError
 
                 with asection(f"Moving array from backend to numpy."):
                     tp_array = Backend.to_numpy(tp_array, dtype=dtype, force_copy=False)
 
-                models[tp] = model.to_numpy()
+                model = model.to_numpy()
+                del views_tp
+                Backend.current().clear_memory_pool()
 
                 aprint(f"Last equalisation ratios: {new_equalisation_ratios}")
-                if equalise_mode == 'first' and tp == 0:
-                    aprint(f"Equalisation mode: 'first' -> saving equalisation ratios: {new_equalisation_ratios} for subsequent time points")
-                    equalisation_ratios_reference[0] = list(float(Backend.to_numpy(ratio)) for ratio in new_equalisation_ratios)
-                elif equalise_mode == 'all':
-                    aprint(f"Equalisation mode: 'all' -> recomputing equalisation ratios for each time point.")
-                    # No need to save, we need to recompute the ratios for each time point.
-                    pass
 
-            if 'fused' not in dest_dataset.channels():
-                try:
+            with asection(f"Saving fused stack for time point {i}, shape:{tp_array.shape}, dtype:{tp_array.dtype}"):
+
+                if i == 0:
+                    # We allocate last minute once we know the shape... because we don't always know the shape in advance!!!
+                    from dexp.datasets.zarr_dataset import ZDataset
+                    mode = 'w' + ('' if overwrite else '-')
+                    dest_dataset = ZDataset(output_path, mode, store, parent=dataset)
                     dest_dataset.add_channel('fused',
-                                             shape=(nb_timepoints,) + tp_array.shape,
-                                             dtype=dtype,
+                                             shape=(len(time_points),)+tp_array.shape,
+                                             dtype=tp_array.dtype,
                                              codec=compression,
                                              clevel=compression_level)
-                except (ContainsArrayError, ContainsGroupError):
-                    aprint(f"Other thread/process created channel before... ")
 
-            with asection(f"Saving fused stack for time point {tp}, shape:{tp_array.shape}, dtype:{tp_array.dtype}"):
                 dest_dataset.write_stack(channel='fused',
-                                         time_point=tp,
+                                         time_point=i,
                                          stack_array=tp_array)
 
-            aprint(f"Done processing time point: {tp}/{nb_timepoints} .")
+            aprint(f"Done processing time point: {i}/{len(time_points)} .")
 
         except Exception as error:
             aprint(error)
-            aprint(f"Error occurred while processing time point {tp} !")
+            aprint(f"Error occurred while processing time point {i} !")
             import traceback
             traceback.print_exc()
 
             if stop_at_exception:
                 raise error
 
-    if workers == -1:
-        workers = len(devices)
-    aprint(f"Number of workers: {workers}")
+        return new_equalisation_ratios, model, dest_dataset
 
-    if workers > 1:
-        Parallel(n_jobs=workers, backend=workersbackend)(delayed(process)(tp, devices[tp % len(devices)], workers) for tp in range(0, nb_timepoints))
+    from dask_cuda import LocalCUDACluster
+    cluster = LocalCUDACluster(CUDA_VISIBLE_DEVICES=devices)
+    client = Client(cluster)
+    aprint('Dask Client', client)
+
+    if microscope == 'simview':
+        init_params = ([None, None, None], None, None)
+    elif microscope == 'mvsols':
+        init_params = ([None], None, None)
     else:
-        for tp in range(0, nb_timepoints):
-            process(tp, devices[0], workers)
+        raise NotImplementedError
+
+    params = process(0, init_params).compute()
+    if equalise_mode == 'all':
+        params = init_params[:2] + params[2]
+
+    lazy_computations = []
+    for i in range(1, len(time_points)):
+        lazy_computations.append(process(i, params))
+
+    models = [model for _, model, _ in dask.compute(*lazy_computations)]
 
     if not loadreg:
         model_list_to_file(model_list_filename, models)
 
+    _, _, dest_dataset = params
+
     aprint(dest_dataset.info())
     if check:
         dest_dataset.check_integrity()
 
-    dest_dataset.set_cli_history(parent=dataset if isinstance(dataset, ZDataset) else None)
     # close destination dataset:
     dest_dataset.close()
