@@ -10,9 +10,9 @@ from dexp.datasets.base_dataset import BaseDataset
 from dexp.processing.backends.backend import Backend
 from dexp.processing.backends.best_backend import BestBackend
 from dexp.processing.backends.numpy_backend import NumpyBackend
-from dexp.processing.registration.sequence_proj import image_stabilisation_proj_
-from dexp.processing.registration.sequence import image_stabilisation
 from dexp.processing.registration.model.sequence_registration_model import SequenceRegistrationModel
+from dexp.processing.registration.sequence import image_stabilisation
+from dexp.processing.registration.sequence_proj import image_stabilisation_proj_
 
 
 def _compute_model(
@@ -108,7 +108,7 @@ def _compute_model(
     return model
 
 
-def dataset_stabilize(input_dataset: BaseDataset,
+def dataset_stabilize(dataset: BaseDataset,
                       output_path: str,
                       channels: Sequence[str],
                       reference_channel: Optional[str] = None,
@@ -144,7 +144,7 @@ def dataset_stabilize(input_dataset: BaseDataset,
 
     Parameters
     ----------
-    input_dataset: Input dataset
+    dataset: Input dataset
     output_path: Output path for Zarr storage
     channels: selected channels
     reference_channel: use this channel to compute the stabilization model and apply to every channel.
@@ -178,14 +178,14 @@ def dataset_stabilize(input_dataset: BaseDataset,
 
     from dexp.datasets.zarr_dataset import ZDataset
     mode = 'w' + ('' if overwrite else '-')
-    output_dataset = ZDataset(output_path, mode, zarr_store)
+    dest_dataset = ZDataset(output_path, mode, zarr_store, parent=dataset)
 
     model = None
     if reference_channel is not None:
-        if reference_channel not in input_dataset.channels():
+        if reference_channel not in dataset.channels():
             raise ValueError(f'Reference channel {reference_channel} not found.')
         model = _compute_model(
-            input_dataset=input_dataset,
+            input_dataset=dataset,
             channel=reference_channel,
             slicing=slicing,
             max_range=max_range,
@@ -206,10 +206,10 @@ def dataset_stabilize(input_dataset: BaseDataset,
             debug_output=debug_output,
             )
 
-    for channel in input_dataset._selected_channels(channels):
+    for channel in dataset._selected_channels(channels):
         if reference_channel is None:
-            model = _compute_model(
-                input_dataset=input_dataset,
+            channel_model = _compute_model(
+                input_dataset=dataset,
                 channel=channel,
                 slicing=slicing,
                 max_range=max_range,
@@ -229,23 +229,36 @@ def dataset_stabilize(input_dataset: BaseDataset,
                 device=device,
                 debug_output=debug_output,
             )
+        else:
+            channel_model = model
+
+        metadata = dataset.get_metadata()
+        if 'dt' in metadata.get(channel, {}):
+            channel_model = channel_model.reduce(int(round(metadata[channel]['dt'])))
+
+        array = dataset.get_array(channel, per_z_slice=False, wrap_with_dask=True)
+        if slicing is not None:
+            array = array[slicing]
 
-        array = input_dataset.get_array(channel, per_z_slice=False, wrap_with_dask=True)
         shape = array.shape
         dtype = array.dtype
-        chunks = ZDataset._default_chunks
         nb_timepoints = shape[0]
 
-        # Shape of the resulting array:
-        padded_shape = (nb_timepoints,) + model.padded_shape(shape[1:])
+        if len(channel_model) != nb_timepoints:
+            aprint(f'WARNING: Number of time points ({nb_timepoints}) and registration models ({len(channel_model)})'
+                   f'does not match. Using the smallest one.')
+            if nb_timepoints > len(channel_model):
+                nb_timepoints = len(channel_model)
+            # else: it is not necessary to change the model sequence length
 
-        # Add channel to output datatset:
-        output_dataset.add_channel(name=channel,
-                                   shape=padded_shape,
-                                   dtype=dtype,
-                                   chunks=chunks,
-                                   codec=compression_codec,
-                                   clevel=compression_level)
+        # Shape of the resulting array:
+        padded_shape = (nb_timepoints,) + channel_model.padded_shape(shape[1:])
+
+        dest_dataset.add_channel(name=channel,
+                                 shape=padded_shape,
+                                 dtype=dtype,
+                                 codec=compression_codec,
+                                 clevel=compression_level)
 
         # definition of function that processes each time point:
         def process(tp):
@@ -254,18 +267,12 @@ def dataset_stabilize(input_dataset: BaseDataset,
                     with asection(f"Loading stack"):
                         tp_array = array[tp].compute()
 
-                    # backend = NumpyBackend() if integral else CupyBackend(device, exclusive=True)
-
                     with NumpyBackend():
-                        xp = Backend.get_xp_module()
-
                         with asection(f"Applying model..."):
-                            # tp_array = Backend.to_backend(tp_array, dtype=array.dtype, force_copy=False)
-                            tp_array = model.apply(tp_array, index=tp, pad=pad, integral=integral)
-                            # tp_array = Backend.to_numpy(tp_array, dtype=array.dtype, force_copy=False)
+                            tp_array = channel_model.apply(tp_array, index=tp, pad=pad, integral=integral)
 
                     with asection(f"Saving stabilized stack for time point {tp}/{nb_timepoints}, shape:{tp_array.shape}, dtype:{array.dtype}"):
-                        output_dataset.write_stack(channel=channel,
+                        dest_dataset.write_stack(channel=channel,
                                                    time_point=tp,
                                                    stack_array=tp_array)
 
@@ -290,10 +297,16 @@ def dataset_stabilize(input_dataset: BaseDataset,
                 process(tp)
 
     # printout output dataset info:
-    aprint(output_dataset.info())
+    aprint(dest_dataset.info())
     if check:
-        output_dataset.check_integrity()
+        dest_dataset.check_integrity()
+
+    # Dataset info:
+    aprint(dest_dataset.info())
+
+    # Check dataset integrity:
+    if check:
+        dest_dataset.check_integrity()
 
-    output_dataset.set_cli_history(parent=input_dataset if isinstance(input_dataset, ZDataset) else None)
     # close destination dataset:
-    output_dataset.close()
+    dest_dataset.close()
