@@ -1,31 +1,27 @@
-import multiprocessing
 import os
 import shutil
 import sys
-from multiprocessing.pool import ThreadPool
-from os.path import isfile, isdir, exists
+from os.path import isfile, isdir, exists, join
 from typing import Tuple, Sequence, Any, Union, Optional
 
 import dask
 import numpy
 import zarr
 from arbol.arbol import aprint
-from numcodecs import blosc
 from zarr import open_group, convenience, CopyError, Blosc, Group
 
 from dexp.datasets.base_dataset import BaseDataset
-# Configure multithreading for Dask:
 from dexp.processing.backends.backend import Backend
-from dexp.utils.config_dask import config_dask
 
-config_dask()
+from dexp.utils.config import config_blosc
+
+
+config_blosc()
 
 
 class ZDataset(BaseDataset):
-    _default_chunks = (1, 128, 512, 512)
-
-    def __init__(self, path: str, mode: str = 'r', store: str = None):
-        """Instanciates a Zarr dataset (and opens it)
+    def __init__(self, path: str, mode: str = 'r', store: str = None, parent: Optional[BaseDataset] = None):
+        """Instantiates a Zarr dataset (and opens it)
 
         Parameters
         ----------
@@ -41,8 +37,6 @@ class ZDataset(BaseDataset):
         Returns
         -------
         Zarr dataset
-
-
         """
 
         super().__init__(dask_backed=False)
@@ -78,7 +72,15 @@ class ZDataset(BaseDataset):
         elif exists(path) and mode == 'w':
             aprint(f"Deleting '{path}' for overwrite!")
             if isdir(path):
-                shutil.rmtree(path, ignore_errors=True)
+                # This is a very dangerous operation, let's double check that the folder really holds a zarr dataset:
+                _zgroup_file = join(path, '.zgroup')
+                _zarray_file = join(path, '.zarray')
+                # We check that either of these two hiddwn files are present, and if '.zarr' is part of the name:
+                if ('.zarr' in path) and (exists(_zgroup_file) or exists(_zarray_file)):
+                    shutil.rmtree(path, ignore_errors=True)
+                else:
+                    raise ValueError("Specified path does not seem to be a zarr dataset, deletion for overwrite not performed out of abundance of caution, check path!")
+
             elif isfile(path):
                 os.remove(path)
 
@@ -121,6 +123,15 @@ class ZDataset(BaseDataset):
         else:
             raise ValueError(f"Invalid read/write mode or invalid path: {path} (check path!)")
 
+        # updating metadata
+        if parent is not None:
+            metadata = parent.get_metadata()
+            metadata.pop('cli_history', None)  # avoiding adding it twice
+            self.append_metadata(metadata)
+
+        if mode in ('a', 'w', 'w-'):
+            self.append_cli_history(parent if isinstance(parent, ZDataset) else None)
+
     def _initialise_existing(self):
         self._channels = [channel for channel, _ in self._root_group.groups()]
 
@@ -147,6 +158,14 @@ class ZDataset(BaseDataset):
         else:
             return groups[0]
 
+    @staticmethod
+    def _default_chunks(shape: Tuple[int], max_size: int = 2 ** 31) -> Tuple[int]:
+        width = shape[-1]
+        height = shape[-2]
+        depth = min(max_size // (width * height), shape[-3])
+        chunk = (1, depth, height, width)
+        return chunk[-len(shape):]
+
     def close(self):
         # We close the store if it exists, i.e. if we have been writing to the dataset
         if self._store is not None:
@@ -186,35 +205,38 @@ class ZDataset(BaseDataset):
 
     def info(self, channel: str = None, cli_history: bool = True) -> str:
         info_str = ''
-        if channel:
+        if channel is not None:
             info_str += f"Channel: '{channel}', nb time points: {self.shape(channel)[0]}, shape: {self.shape(channel)[1:]}"
-            info_str += "\n"
+            info_str += ".\n"
             info_str += str(self._arrays[channel].info)
             return info_str
         else:
-            info_str += f"Dataset at location: {self._path}"
-            info_str = "Zarr tree: \n"
+            info_str += f"Dataset at location: {self._path} \n"
+            info_str += f"Channels: {self.channels()} \n"
+            info_str += "Zarr tree: \n"
             info_str += str(self._root_group.tree())
-            info_str += "\n\n"
-            info_str += "Channels: \n"
-            for channel in self.channels():
-                info_str += "  └──" + self.info(channel) + "\n\n"
+            info_str += ".\n\n"
+            info_str += "Arrays: \n"
+            for name, array in self._arrays.items():
+                info_str += "  │ \n"
+                info_str += "  └──" + name +":\n" + str(array.info) + "\n\n"
+                info_str += ".\n\n"
 
-        info_str += "\n\n"
-        info_str += "\n\n"
-        for name, array in self._arrays.items():
-            info_str += f"Channel: {name}  \n"
-            info_str += str(array.info)
-            info_str += '\n'
+        info_str += ".\n\n"
+        info_str += f"\nMetadata: \n"
+        for key, value in self.get_metadata().items():
+            if 'cli_history' not in key:
+                info_str += f"\t{key} : {value} \n"
 
         if cli_history:
+            info_str += ".\n\n"
             key = 'cli_history'
             if key in self._root_group.attrs:
-                info_str += "\nCommand line history:\n/\n"
+                info_str += "\nCommand line history:\n"
                 commands_list = self._root_group.attrs[key]
                 for command in commands_list[:-1]:
-                    info_str += " ├──" + command + "\n"
-                info_str += " └──" + commands_list[-1] + "\n"
+                    info_str += " ├──■ '" + command + "' \n"
+                info_str += " └──■ '" + commands_list[-1] + "' \n"
 
         return info_str
 
@@ -225,7 +247,10 @@ class ZDataset(BaseDataset):
             attrs[name] = self._root_group.attrs[name]
         return attrs
 
-    def set_cli_history(self, parent: Optional['ZDataset']):
+    def append_metadata(self, metadata: dict):
+        self._root_group.attrs.update(metadata)
+
+    def append_cli_history(self, parent: Optional[BaseDataset]):
         key = 'cli_history'
         cli_history = []
         if parent is not None:
@@ -310,7 +335,7 @@ class ZDataset(BaseDataset):
             raise ValueError("Channel already exist!")
 
         if chunks is None:
-            chunks = ZDataset._default_chunks[0: len(shape)]
+            chunks = self._default_chunks(shape)
 
         aprint(f"chunks={chunks}")
 
