@@ -5,6 +5,7 @@ import scipy
 from dask.array import Array
 
 from dexp.processing.backends.backend import Backend
+from dexp.utils import xpArray
 
 
 class NumpyBackend(Backend):
@@ -34,7 +35,7 @@ class NumpyBackend(Backend):
     def clear_memory_pool(self):
         pass
 
-    def _to_numpy(self, array, dtype=None, force_copy: bool = False) -> numpy.ndarray:
+    def _to_numpy(self, array: xpArray, dtype=None, force_copy: bool = False) -> numpy.ndarray:
 
         if isinstance(array, Array):
             return self._to_numpy(array.compute(), dtype=dtype, force_copy=force_copy)
@@ -46,7 +47,7 @@ class NumpyBackend(Backend):
         else:
             return numpy.asarray(array)
 
-    def _to_backend(self, array, dtype=None, force_copy: bool = False) -> Any:
+    def _to_backend(self, array: xpArray, dtype=None, force_copy: bool = False) -> Any:
 
         if isinstance(array, Array):
             return self._to_backend(array.compute(), dtype=dtype, force_copy=force_copy)
@@ -58,7 +59,7 @@ class NumpyBackend(Backend):
         else:
             return array
 
-    def _get_xp_module(self, array=None) -> Any:
+    def _get_xp_module(self, array: xpArray=None) -> Any:
         if array is None:
             return numpy
         else:
@@ -68,7 +69,7 @@ class NumpyBackend(Backend):
             except ModuleNotFoundError:
                 return numpy
 
-    def _get_sp_module(self, array=None) -> Any:
+    def _get_sp_module(self, array: xpArray=None) -> Any:
         if array is None:
             return scipy
         else:
