@@ -12,17 +12,19 @@ from dexp.processing.filters.butterworth_filter import butterworth_filter
 from dexp.processing.fusion.dct_fusion import fuse_dct_nd
 from dexp.processing.fusion.dft_fusion import fuse_dft_nd
 from dexp.processing.fusion.tg_fusion import fuse_tg_nd
-from dexp.processing.multiview_lightsheet.fusion.simview import fuse_illumination_views
+from dexp.processing.multiview_lightsheet.fusion.simview import SimViewFusion
 from dexp.processing.registration.model.pairwise_registration_model import PairwiseRegistrationModel
 from dexp.processing.registration.translation_nd import register_translation_nd
 from dexp.processing.registration.translation_nd_proj import register_translation_proj_nd
 from dexp.processing.registration.warp_multiscale_nd import register_warp_multiscale_nd
 from dexp.processing.restoration.clean_dark_regions import clean_dark_regions
 from dexp.processing.restoration.dehazing import dehaze
+from dexp.utils import xpArray
 
 
 @section("mvSOLS 1C2L fusion")
-def msols_fuse_1C2L(C0L0, C0L1,
+def msols_fuse_1C2L(C0L0: xpArray,
+                    C0L1: xpArray,
                     dz: float,
                     dx: float,
                     angle: float,
@@ -297,10 +299,10 @@ def msols_fuse_1C2L(C0L0, C0L1,
     #     viewer.add_image(_c(C0L1), name='C0L1', colormap='bop orange', blending='additive')
 
     with asection(f"Fuse detection views C0lx and C1Lx..."):
-        C1Lx = fuse_illumination_views(C0L0, C0L1,
-                                       mode=fusion,
-                                       bias_exponent=fusion_bias_exponent,
-                                       bias_strength=fusion_bias_strength_x)
+        C1Lx = SimViewFusion._fuse_views_generic(C0L0, C0L1, bias_axis=2, mode=fusion,
+                                                 bias_exponent=fusion_bias_exponent,
+                                                 bias_strength=fusion_bias_strength_x,
+                                                 smoothing=12)
         Backend.current().clear_memory_pool()
 
     if dehaze_size > 0 and not dehaze_before_fusion:
@@ -361,6 +363,8 @@ def fuse_illumination_views(CxL0, CxL1,
         fused = fuse_dct_nd(CxL0, CxL1)
     elif mode == 'dft':
         fused = fuse_dft_nd(CxL0, CxL1)
+    else:
+        raise NotImplementedError
 
     return fused
 
@@ -373,7 +377,9 @@ def resample_C0L0(image,
                   num_split: int = 8):
     if mode == 'yang':
         return yang_deskew(image,
-                           flip=True,
+                           depth_axis=0,
+                           lateral_axis=1,
+                           flip_depth_axis=True,
                            dx=dx,
                            dz=dz,
                            angle=angle,
@@ -390,7 +396,9 @@ def resample_C0L1(image,
                   num_split: int = 8):
     if mode == 'yang':
         return yang_deskew(image,
-                           flip=False,
+                           depth_axis=0,
+                           lateral_axis=1,
+                           flip_depth_axis=False,
                            dx=dx,
                            dz=dz,
                            angle=angle,
